from pathlib import Path
from graph_articulations.config import *
import pandas as pd
import simplejson as json
import argparse
import shutil

parser = argparse.ArgumentParser(description='Get model ids by category')
parser.add_argument('--out_path', default=None, help='path of desired output json')
parser.add_argument('--models_csv', default=None, help='path to scene-toolkit/server/static/data/suncg/suncg.planner5d.models.full.csv')
parser.add_argument('--category_json', default=None, help='path to a json list of categories')
parser.add_argument('--docker_home', default='/home/ham', help='path to desired working dir')
parser.add_argument('--model_root', default=None, help='path to model meshes')
parser.add_argument('--part_root', default=None, help='path to parts')
parser.add_argument('--conn_root', default=None, help='path to connectivity graphs')

args = parser.parse_args()

docker_home = Path(args.docker_home)
home = docker_home if docker_home.exists() else Path.home()

json_output_path = GRAPH_ART_ROOT / 'category_to_models.json' if args.out_path is None else args.out_path
model_csv_path = home / 'scene-toolkit/server/static/data/suncg/suncg.planner5d.models.full.csv' if args.models_csv is None else args.model_csv_path
categories_json_path = GRAPH_ART_ROOT / 'categories.json' if args.category_json is None else args.category_json

models_df = pd.read_csv(model_csv_path)

# List of categories to collect model ids for
categories = None
with open(categories_json_path, 'r') as categories_json: categories = json.load(categories_json)


def get_models_from_category(category_name: str):
    """
    category_name: Name of the category that we want to collect the models ids for
    
    Returns: List of model ids that belong to category_name
    """
    index_where_category = models_df.category.str.contains(category_name)
    category_indexes = models_df.index[index_where_category]
    category_df = models_df.iloc[category_indexes]
    return category_df.id.values.tolist()


def copy_all_model_info(target_path: str = None):
    """
    Use to move data from separate directories to one place
    :param target_path: Destination path of copy
    """
    if target_path is None:
        target_path = GRAPH_ART_ROOT / 'articulation_data'

    target_path = Path(target_path)
    if not target_path.exists(): Path.mkdir(target_path)

    for category, models in category_to_models.items():
        for model in models:
            # if model == "s__788":
            #     continue

            model_suffix = f'/{model}.obj'
            part_suffix = f'/{model}.parts.json'
            conn_suffix = f'/{model}.artpre.json'

            model_path = str(MODEL_ROOT / model) + model_suffix
            part_path = str(PART_ROOT / model) + part_suffix
            connectivity_path = str(CONNECTIVITY_ROOT) + conn_suffix
            # if Path(model_path).exists() and Path(part_path) and Path(conn_suffix):
            shutil.copy(model_path, str(target_path ))
            shutil.copy(part_path, str(target_path ))
            shutil.copy(connectivity_path, str(target_path ))


if __name__ == '__main__':
    part_root = PART_ROOT if args.part_root is None else args.part_root
    connectivity_root = CONNECTIVITY_ROOT if args.conn_root is None else args.conn_root
    model_root = MODEL_ROOT if args.model_root is None else args.model_root

    # Iterate through category and store all outputs in a comprehensive dictionary
    # Dump this dictionary to JSON for future use
    category_to_models = {cur_category: get_models_from_category(cur_category) for cur_category in categories}
    with open(json_output_path, 'w') as json_file:
        json.dump(category_to_models, json_file)

    # Relevant paths for copying function
    # Copy all relevant info to a single directory
    # copy_all_model_info()



import open3d as o3d
import numpy as np
import torch
from graph_articulations.data.process_models import find_part_instances

# Colors
# Refer to http://doc.instantreality.org/tools/color_calculator/
red = np.array([1, 0, 0])
orange = np.array([1, 0.5, 0])
yellow = np.array([1, 1, 0])
green = np.array([0.337, 0.890, 0.329])
blue = np.array([0, 1, 0])
indigo = np.array([0.294, 0, .51])
violet = np.array([.541, .169, .886])
black = np.array([0.027, 0.031, 0.027])
pink = np.array([0.988, 0.352, 0.984])


def bounding_boxes_lines(bboxes):
    """
    :param bboxes: Nd array with shape (# of Pid's, 8, 3)
    :return: LineSet Geometry of All bounding boxes for the current model
    """

    # Placeholder Lineset
    bounding_boxes = o3d.geometry.LineSet()

    # Iterate through the bounding boxes and construct lines
    for bb in range(bboxes.shape[0]):
        cur_box = bboxes[bb][:]

        # Connect the vertices
        # Color the lines Red
        lines = [[0, 1], [2, 3], [4, 5], [6, 7], [0, 4], [2, 6], [1, 5], [3, 7], [0, 3], [1, 2], [4, 7], [5, 6]]
        colors = [[1, 0, 0] for line in range(len(lines))]

        # Construct the current lineSet
        line_set = o3d.geometry.LineSet()
        line_set.points = o3d.utility.Vector3dVector(cur_box)
        line_set.lines = o3d.utility.Vector2iVector(lines)
        line_set.colors = o3d.utility.Vector3dVector(colors)

        # Add the current bound box to the collective set
        bounding_boxes = bounding_boxes + line_set
    return bounding_boxes


def part_points(points, labels, object_instances, colors=None, scale=None, part_instances:[]=None):
    """
    :param data: graph in the dataset
    :param colors: colors to set
    :return: Colored Point Cloud Object to Render
    """

    if colors is None:
        colors = np.ones(points.shape) * orange

    cabinet_index = np.where(labels == object_instances.index("cabinet"))
    cabinet_door_index = np.where(labels == object_instances.index("cabinet door"))
    door_index = np.where(labels == object_instances.index("door"))
    drawer_index = np.where(labels == object_instances.index("drawer"))
    side_panel_index = np.where(labels == object_instances.index("side panel"))
    top_panel_index = np.where(labels == object_instances.index("top panel"))
    unknown_index = np.where(labels == object_instances.index("unknown"))

    colors[drawer_index] = indigo
    colors[cabinet_index] = black
    colors[cabinet_door_index] = green
    colors[door_index] = green
    colors[side_panel_index] = red
    colors[top_panel_index] = yellow
    colors[unknown_index] = pink

    if scale is not None:
        points *= scale


    pcd = o3d.geometry.PointCloud()
    pcd.points = o3d.utility.Vector3dVector(points.numpy())
    pcd.colors = o3d.utility.Vector3dVector(colors)
    return pcd


def tensor_to_point_cloud(data, object_instances, colors=None, scale=None, part_instances:[]=None):
    """
    :param data: graph in the dataset
    :param colors: colors to set
    :return: Colored Point Cloud Object to Render
    """
    points = data.pos
    labels = data.y

    if colors is None:
        colors = np.ones(points.shape) * orange

    cabinet_index = np.where(labels == object_instances.index("cabinet"))
    cabinet_door_index = np.where(labels == object_instances.index("cabinet door"))
    door_index = np.where(labels == object_instances.index("door"))
    drawer_index = np.where(labels == object_instances.index("drawer"))
    side_panel_index = np.where(labels == object_instances.index("side panel"))
    top_panel_index = np.where(labels == object_instances.index("top panel"))
    unknown_index = np.where(labels == object_instances.index("unknown"))

    colors[drawer_index] = indigo
    colors[cabinet_index] = black
    colors[cabinet_door_index] = green
    colors[door_index] = green
    colors[side_panel_index] = red
    colors[top_panel_index] = yellow
    colors[unknown_index] = pink

    if scale is not None:
        points *= scale



    pcd = o3d.geometry.PointCloud()
    pcd.points = o3d.utility.Vector3dVector(points.numpy())
    pcd.colors = o3d.utility.Vector3dVector(colors)
    return pcd


def show_each_part_separately(data: object, object_instance: list):
    """
    :param data: Graph containing model information
    :param object_instance: list of all parts in the dataset
    :return: Renders each part-instance separately
    """

    for idx in range(len(data.pids.keys)):
        points = data.parts[f'{idx}']
        label, inst = data.pids[f'{idx}'][0], data.pids[f'{idx}'][1]
        labels = np.ones(points.shape[0]) * label

        print(f'{object_instance[label]}-{inst} has {points.shape} points')

        pcd = part_points(points, labels, object_instances=object_instance)
        bbox = bounding_boxes_lines(data.bounding_boxes)
        o3d.draw_geometries([bbox, pcd])


if __name__ == '__main__':
    from torch_geometric.data import Data
    from models import PointNetSegEncoder, PointNetEncoder
    from graph_articulations.config import GRAPH_ART_ROOT

    import torch.optim as optim


                # with torch.no_grad():
                #     # TODO get feature embedding here
                #
                #
                # # TODO compute distances and construct connectivity graph here
                # threshold = 0.5
                #
                # for
                #
                # else:
                # points = points[part_indices]
                # labels = labels[part_indices]
                #
                # pcd = part_points(points, labels, object_instances=object_instance)
                # bbox = bounding_boxes_lines(bboxes)
                # o3d.draw_geometries([bbox, pcd])

    model = PointNetSegEncoder(num_classes=4)
    optim = torch.optim.Adam(model.parameters(), lr=0.001)
    checkpoint = torch.load(GRAPH_ART_ROOT / 'PointNetSeg')
    model.load_state_dict(checkpoint['state_dict'], strict=False)
    optim.load_state_dict(checkpoint['optimizer'])
    criterion = checkpoint['loss']
    print(f'{checkpoint.keys()}')
    keys = list(checkpoint.keys())
    print(f'{"epoch" in checkpoint.keys()}')

    # print(model)

    # part_data = [Data(pos=points)]
    #
    # model.eval()
    #
    # with torch.no_grad():
    #     for part in part_data:
    #         points = part.pos
    #         output = model(points)

        # TODO how to iterate through by part dataset




from graph_articulations.data import update_part_and_instance_labels
from torch_geometric.data import Data
import torch
import numpy as np


class SamplePointsWithLabels(object):
    r"""Uniformly samples :obj:`num` points on the mesh faces according to
    their face area.

    Args:
        num (int): The number of points to sample.
        remove_faces (bool, optional): If set to :obj:`False`, the face tensor
            will not be removed. (default: :obj:`True`)
        include_normals (bool, optional): If set to :obj:`True`, then compute
            normals for each sampled point. (default: :obj:`False`)
    """

    def __init__(self, num, remove_faces=True, include_normals=False, object_instances=None,
                 categorical_instances=None, display=False):
        self.num = num
        self.remove_faces = remove_faces
        self.include_normals = include_normals
        self.object_instances = object_instances
        self.categorical_instances = categorical_instances
        self.display = display

    def __call__(self, data):
        pos, face, model_name = data.pos, data.face, data.name
        assert pos.size(1) == 3 and face.size(0) == 3

        # Get face part and instance labels
        face_part_labels, face_instance_labels, pid_points_indices = \
            update_part_and_instance_labels(self.categorical_instances[model_name], self.object_instances)

        # Construct sub graph of pid and corresponding point indices
        # Add the subgraph to the current model's data object
        pid_graph = Data.from_dict(pid_points_indices)
        data.__setitem__('pids', pid_graph)

        # Movable parts of interest before sample
        if self.display:
            print(f'Part labels shape: {face_part_labels.shape}')
            drawer_count = np.where(face_part_labels == 0)[0]
            cabinet_count = np.where(face_part_labels == 7)[0]
            print(f'Drawer count: {drawer_count.shape}')
            print(f'Cabinet count: {cabinet_count.shape}')

        # Normalize vertices
        pos_max = pos.max()
        pos = pos / pos_max

        #  The cross product between to the two edges to get the area in between
        #  Normalize the surface area
        area = (pos[face[1]] - pos[face[0]]).cross(pos[face[2]] - pos[face[0]])
        area = area.norm(p=2, dim=1).abs() / 2

        # Get of sampling probability for each face
        prob = area / area.sum()
        sample = torch.multinomial(prob, self.num, replacement=True)

        # Sample from faces, face part labels and face instance labels
        face = face[:, sample]
        face_part_labels = face_part_labels[sample]
        face_instance_labels = face_instance_labels[sample]

        # Movable parts of interest after sample
        if self.display:
            drawer_count = np.where(face_part_labels == 0)[0]
            cabinet_count = np.where(face_part_labels == 7)[0]
            print(f'Part labels shape: {face_part_labels.shape}')
            print(f'Drawer count: {drawer_count.shape}')
            print(f'Cabinet count: {cabinet_count.shape}')

        frac = torch.rand(self.num, 2, device=pos.device)
        mask = frac.sum(dim=-1) > 1
        frac[mask] = 1 - frac[mask]

        vec1 = pos[face[1]] - pos[face[0]]
        vec2 = pos[face[2]] - pos[face[0]]

        if self.include_normals:
            data.norm = torch.nn.functional.normalize(vec1.cross(vec2), p=2)

        pos_sampled = pos[face[0]]
        pos_sampled += frac[:, :1] * vec1
        pos_sampled += frac[:, 1:] * vec2

        pos_sampled = pos_sampled * pos_max

        # Add samples as attributes to the data object
        data.pos = pos_sampled
        data.__setitem__('y', face_part_labels)
        data.__setitem__('instance_label', face_instance_labels)

        from collections import OrderedDict
        part_points = OrderedDict()
        # Get individual point parts
        for pid in range(len(data.pids)):
            part_label, inst_label = data.pids[str(pid)][0], data.pids[str(pid)][1]
            label_indices = np.where(data.y == part_label)
            inst_indices = np.where(data.instance_label[label_indices] == inst_label)
            points = pos_sampled[label_indices]
            points = points[inst_indices]
            part_points[str(pid)] = points

        part_graph = Data.from_dict(part_points)
        data.__setitem__('parts', part_graph)

        if self.remove_faces:
            data.face = None

        return data



from pathlib import Path
from graph_articulations.config import GRAPH_ART_ROOT
from graph_articulations.data import categorical_list_of_models
from graph_articulations import list_of_models_with_issues, line_br
import os
import simplejson as json



def find_mpid_source_combo(sim_dir=None, input_cat=None, output_path=None):
    """
    Writes comprehensive nested dictionary { (key) Model : (value) { (key) mpid : (value) [source_mpid1, ... , source_mpidn]}
    path_to_test_models = [Path(sim_dir, model) for model in test_set] to json file.

    :param sim_dir: root path of model similarity information. If not provide, then uses ARTICULATIONS_SIM_DIR variable
    :param eval_dir: path to the evaluation directory
    :param output_path: path to write comprehensive json file too
    """

    if sim_dir is None:
        sim_dir = os.environ["ARTICULATIONS_SIM_DIR"]
        assert sim_dir is not None, 'ARTICULATIONS_SIM_DIR is not set as an environment variable'

    if input_cat is None: cat_dict = categorical_list_of_models(input_cat)
    keys = [k for k, _ in cat_dict.items()]

    # First level of nested dictionary { (key) Model : (value) { (key) mpid : (value) [source_mpid1, ... , source_mpidn]}
    path_to_test_models = [Path(sim_dir, model) for key in keys for model in cat_dict[key]]
    mpid_dict = {}

    bad_models = list_of_models_with_issues()
    skipped = []

    # Iterate through every subdirectory that test models posses
    for model_path in path_to_test_models:
        model = model_path.name
        mpid_dict[model] = []
        mpids_source_mpids = []

        if model in bad_models:
            print(f'Skipping {model}.... it has been blacklisted')
            skipped.append(model)
            continue

        for part_path in model_path.iterdir():
            mpid = model + '-' + part_path.name
            print(f'Getting sources for mpid: {mpid}...')

            for source_model in part_path.iterdir():
                # Find every json file that references source models
                # Read pid from current file
                with source_model.open('r') as read_file:
                    source_info = json.load(read_file)
                    read_file.close()

                # Get mpid and source_mpid from path
                source_mpid = source_model.name[:-5] + '-' + str(source_info[-1]['pid2'])
                mpids_source_mpids.append(source_mpid)

            mpid_dict[model].append({mpid: mpids_source_mpids})

    if output_path is None:
        output_path = GRAPH_ART_ROOT / 'considered_mpids_and_source_mpids.json'

    with open(output_path, 'w') as json_file:
        json.dump(mpid_dict, json_file)
        json_file.close()

    print(f'Skipped the following: \n{line_br}\n{skipped}')
    print('Done :)')


if __name__ == '__main__':
    find_mpid_source_combo()



from pathlib import Path
import os

PART_ROOT = Path(os.environ['ARTICULATIONS_PARTS_DIR'])
CONNECTIVITY_ROOT = Path(os.environ['ARTICULATIONS_GRAPH_DIR'])
MODEL_ROOT = Path(os.environ['ARTICULATIONS_MESH_DIR'])
GRAPH_ART_ROOT = Path(os.path.dirname(os.path.abspath(__file__))).parent.parent
DATA_ROOT = GRAPH_ART_ROOT / 'articulation_data'



import open3d as o3d
from torch_geometric.data import Dataset
from pathlib import Path
from graph_articulations import obj_to_ply_to_shell_script, list_of_models_with_issues, line_br
from graph_articulations.data import get_con_graph_and_bounding_boxes, aggregate_part_labels, SamplePointsWithLabels
from graph_articulations.config import MODEL_ROOT, DATA_ROOT
from graph_articulations.visualization import bounding_boxes_lines, tensor_to_point_cloud
import torch
import torch_geometric.read as read
import torch_geometric.transforms as T
import tqdm
import time

class Articulations(Dataset):
    """
    .. note::
        Data objects hold mesh faces instead of edge indices.
        To convert the mesh to a graph, use the
        :obj:`torch_geometric.transforms.FaceToEdge` as :obj:`pre_transform`.
        To convert the mesh to a point cloud, use the
        :obj:`torch_geometric.transforms.SamplePoints` as :obj:`transform` to
        sample a fixed number of points on the mesh faces according to their
        face area.
    """

    def __init__(self, root, categorical_instances, transform=None, pre_transform=None, visualize=False):
        super(Articulations, self).__init__(root, transform, pre_transform)
        self.root = root
        self.visualize = visualize
        self.known_parts = aggregate_part_labels(categorical_instances)
        self.categorical_instances = categorical_instances
        self.num_classes = len(self.known_parts)
        self.model_names = []
        self.process()


    def _process(self):
        pass

    @property
    def names(self):
        return self.model_names

    @property
    def raw_file_names(self):
        return [model + '.ply' for model in self.categorical_instances]

    # @property
    # def processed_file_names(self):
    #     return self.processed_file_names()

    def __len__(self):
        return len(self.processed_file_names()) - len(list_of_models_with_issues())

    def _download(self):
        pass
        # Download to `self.raw_dir`.

    def process(self):
        i = 0
        bad_models = list_of_models_with_issues()


        if not Path((self.raw_paths[-1])).exists():
            print(f'Found {MODEL_ROOT}! Moving models to {DATA_ROOT}')
            obj_to_ply_to_shell_script(self.categorical_instances)

        print(f'Loading Dataset \n{line_br}')
        for raw_path in tqdm.tqdm(self.raw_paths):
            # Read data from `raw_path`.
            time.sleep(0.01)
            model_name = Path(raw_path).name.split('.')[0]
            data = read.read_ply(raw_path)
            data.__setitem__('name', i)

            # TODO Let Angel know of this issue
            if model_name in bad_models:
                continue

            self.model_names.append(self.categorical_instances[i])
            con_graph, bboxs, origin = get_con_graph_and_bounding_boxes(model_name, self.known_parts)
            data.__setitem__('connectivity_graph', con_graph)
            data.__setitem__('bounding_boxes', bboxs)
            data.__setitem__('origin', origin)

            if self.pre_filter is not None and not self.pre_filter(data):
                continue

            if self.pre_transform is not None:
                data = self.pre_transform(data)

            # GUI Display of each model in the dataset
            if self.visualize:
                print(data.name)
                pcd = tensor_to_point_cloud(data, object_instances=self.known_parts)
                bboxes = bounding_boxes_lines(bboxs)
                o3d.draw_geometries([bboxes, pcd])

            if not Path(self.processed_dir).exists():
                Path.mkdir(Path(self.processed_dir))
            torch.save(data, Path(self.processed_dir, f'data_{i}.pt'))
            i += 1

    def __getitem__(self, item):
        data = self.get(item)
        data = data if self.transform is None else self.transform(data)
        return data

    def get(self, idx):
        data = torch.load(Path(self.processed_dir, f'data_{idx}.pt'))
        print(f'Loaded {self.names[idx]}')
        return data
        # cur_count = 0
        # for path in Path(self.processed_dir).iterdir():
        #     if cur_count == idx:
        #         data = torch.load(path)
        #         return data
        #     cur_count += 1

    def processed_file_names(self):
        return [model + '.pt' for model in self.categorical_instances]




from pathlib import Path
from bbox import BBox3D
from collections import OrderedDict
from graph_articulations.config import *
from graph_articulations import line_br, obj_to_ply_to_shell_script
from torch import from_numpy, tensor
import argparse
import numpy as np
import simplejson as json
import torch


def aggregate_part_labels(list_of_all_models, display=False):
    """
    Given a list of strings model names, this function returns a sorted list of parts
    :param list_of_all_models: List of models to find part information for
    :return master_part_list: Sorted list of parts from Highest to lowest frequency of occurrence
    """
    part_count_dict = {}
    for model in list_of_all_models:
        parts_path = str(PART_ROOT / model / model) + '.parts.json'
        with open(parts_path, 'r') as part_info:
            part_json = json.load(part_info)

        part_instances = part_json['segmentation'][6]['labels']
        for clss in part_instances:
            part_count_dict[clss] = part_count_dict[clss] + 1 if clss in part_count_dict.keys() else 1

    # Sort parts by number of occurrence in the dataset (Highest to lowest)
    # Get only the part class name from the sorted results
    master_part_list = [part[0] for part in sorted(part_count_dict.items(), key=lambda kv: kv[1], reverse=True)]
    list_of_tuples = [(key, part_count_dict[key]) for key in master_part_list]
    part_count_dict = OrderedDict(list_of_tuples)

    if display: print(f'Found Part Classes by Occurrence (Highest to lowest)\n{line_br}\n{part_count_dict}')
    return master_part_list


def update_part_and_instance_labels(model: str, object_instances=None, display=False):
    """
    :param object_instances: list of comprehensive parts in the dataset
    :param model: (String) model id. (e.g "100", 's__669', etc). Does not expect 'p5d.' as prefix
    :return: Updated global part labels, local instance labels, updated pid to point labels
    """
    # Get path to the model and part information
    # Read the json into memory
    parts_path = str(PART_ROOT / model / model) + '.parts.json'
    with open(parts_path, 'r') as part_info: part_json = json.load(part_info)

    # Grab the part label names and indices
    json_face_labels = np.array(part_json['segmentation'][6]['index'])
    json_label_names = np.array(part_json['segmentation'][6]['labels'])

    # Out-of-place variables ... keep things simple you babo
    # Store the instance of the part as well (e.g drawer 1, drawer 2, etc)
    # Part Labels
    instance_labels = np.zeros(json_face_labels.shape, dtype=np.int64)
    part_labels = np.zeros(json_face_labels.shape, dtype=np.int64)

    if display: print(f'Updating {model}\'s part labels...\n{line_br}')


    # Changing current indices to match the comprehensive class list
    pid_to_points_dict = OrderedDict()
    for index in range(len(json_label_names)):
        new_label = object_instances.index(json_label_names[index])
        label_name = json_label_names[index]

        # Get the number of instances so far for the current part classes
        unique, instance_counts = np.unique(json_label_names[:index + 1], return_counts=True)
        cur_instance = instance_counts[np.where(unique == label_name)][0]

        # Get the current face indices of the current part
        # Update the part label and instance label accordingly
        indices = np.where(json_face_labels == index)

        # Update the Labels out-of-place
        part_labels[indices] = new_label
        instance_labels[indices] = cur_instance
        pid_to_points_dict[str(index)] = (new_label, cur_instance)

        # Display
        if display: print(f'{index}) {label_name}-{cur_instance} encoded from {index} ===> {new_label}')
        # print(f'Index {index} | Unique: {unique} | Instance #: {instance_counts} | cur_instance {cur_instance}')
        # print(f'{len(part_labels[indices])} changed from {index} to {new_label}')
    return from_numpy(part_labels), from_numpy(instance_labels), pid_to_points_dict


def get_con_graph_and_bounding_boxes(model: str, part_labels: list, display: bool = False):
    """
    :param model: name of the model
    :param part_labels: list of all known part labels in the dataset
    :param display: if True prints progress out to the console
    :return: A model's Connectivity Graphs, Bounding Boxes and Name of Parts
    """
    parts_path = str(PART_ROOT / model / model) + '.parts.json'
    conn_path = str(CONNECTIVITY_ROOT) + f'/{model}.artpre.json'

    with open(parts_path, 'r') as part_info:
        part_json = json.load(part_info)
    with open(conn_path, 'r') as json_file:
        precom_dict = json.load(json_file)

    # Grab the part label names and indices
    label_names = np.array(part_json['segmentation'][6]['labels'])
    con_graph = precom_dict['connectivityGraph']
    distance_matrix = np.array(precom_dict['distanceMatrix'])
    updated_graph = np.zeros((len(label_names), len(label_names)))
    updated_dis = np.zeros((updated_graph.shape))
    updated_graph[:][:] = np.nan
    updated_dis[:][:] = np.nan

    # Update Connectivity Graph
    for cur_part in range(len(con_graph)):
        orig_idx = []
        dis = []
        not_none_indices = []
        cur_dis = np.array(distance_matrix[cur_part])

        # Todo confirm distance the provide part distances... not very clear to me at the moment
        if cur_dis.size != 0:
                not_none_indices = np.where(cur_dis != None)
                dis = cur_dis[not_none_indices]
                if not_none_indices[0].size != 0:
                    updated_dis[cur_part][not_none_indices] = np.around((dis * 10).astype(float), decimals=6)
                    not_none_indices = not_none_indices[0]
        if display:
            print(f'\n{cur_part}) {label_names[cur_part]} - {con_graph[cur_part]} \n{line_br}'
                          # f'\n Original Distances: {distance_matrix[cur_part]}\nUpdated Distances: {updated_dis[cur_part]}'
                          # f'\nLength check: {len(not_none_indices)} - {len(con_graph[cur_part])}'
                          )

        for node in con_graph[cur_part]:
            index = node
            orig_idx.append(orig_idx)
            new_label = part_labels.index(label_names[index])
            updated_graph[cur_part][index] = new_label
            if display:
                print(f'{index}) {label_names[node]} encoded from {index} ===> {new_label}'
                      f'\nUpdated graph: {updated_graph[cur_part]}')

    if display:
        print(updated_graph)

    # Store the JSON into placeholder variables
    part_obb_dict = precom_dict['parts']
    names = np.array([])
    centroids = np.array([])
    axes_lengths = np.array([])
    normalized_axes = np.array([])
    pids = np.array([])

    # Populate arrays with json information
    for obb in part_obb_dict:
        if obb is not None:
            if pids.size == 0:
                names = np.array(obb['name'])
                centroids = np.array(obb['obb']['centroid'])
                axes_lengths = np.array(obb['obb']['axesLengths'])
                normalized_axes = np.array(obb['obb']['normalizedAxes'])
                pids = np.array(obb['pid'], dtype=int)
                if 'side' in obb["name"]:
                    if display: print(f'Swapped axes on pid {obb["pid"]}: {obb["name"]}')
                    axes_lengths = np.array([axes_lengths[-1], axes_lengths[1], axes_lengths[0]])
            else:
                names = np.vstack((names, obb['name']))
                pids = np.vstack((pids, np.array(obb['pid'])))
                centroids = np.vstack((centroids, np.array(obb['obb']['centroid'])))
                axes_lengths = np.vstack((axes_lengths, np.array(obb['obb']['axesLengths'])))
                normalized_axes = np.vstack((normalized_axes, np.array(obb['obb']['normalizedAxes'])))

                if 'side' in obb["name"]:
                    if display: print(f'Swapped axes on pid {obb["pid"]}: {obb["name"]}')
                    axes_lengths[-1] = np.array([axes_lengths[-1][-1], axes_lengths[-1][1], axes_lengths[-1][0]])

    # Compute Bounding Boxes
    b_boxes = np.array([])
    for part in range(len(pids)):
        x, y, z = centroids[part][0], centroids[part][1], centroids[part][2]
        l, w, h = axes_lengths[part][0], axes_lengths[part][1], axes_lengths[part][2]
        bbox = BBox3D(x, y, z, length=l, width=w, height=h)
        b_boxes = bbox.p if b_boxes.size == 0 else np.vstack((b_boxes, bbox.p))
    b_boxes = np.reshape(b_boxes, (len(pids), 8, 3))

    return from_numpy(updated_graph), from_numpy(b_boxes), centroids #, from_numpy(updated_dis)


def categorical_list_of_models(path_to_category_list=None):
    """
    :param path_to_category_list: path to the json file that details models for each category
    :return: Dictionary from the loaded category json file
    """
    if path_to_category_list is None:
        path_to_category_list = GRAPH_ART_ROOT / 'category_to_models.json'
    with open(path_to_category_list, 'r') as file:
        categorical_instances = json.load(file)
    return categorical_instances


def find_part_instances(data, object_instances:list=None, display=False):
    """
    :param data: Graph data object with model information
    :param object_instances: list of all
    :return: Consolidated Tensor of Encode Part Labels and Instances
    """
    unique_parts = torch.unique(data.y)
    part_instances = torch.zeros(unique_parts.shape, dtype=torch.int8)
    part_names = [object_instances[part] for part in unique_parts]
    part_indices = [torch.where(data.y == part) for part in unique_parts]
    if display: print(f'\nModel part-instance stats\n{line_br}')
    for cur_part in range(len(part_indices)):
        num_of_instances = torch.unique(data.instance_label[part_indices[cur_part]])
        if display: print(f'{part_names[cur_part]} has {len(num_of_instances)} instances')
        part_instances[cur_part] = len(num_of_instances)
    return unique_parts, part_instances


def list_of_separated_parts(data: object, object_instance: list, display=False):
    """
    :param data: Data object containing the current model's information
    :param object_instance: List of all parts within the dataset
    :return: Two lists of tensors representing each part's points position and labels for the current model
    """
    collective_part_points = []
    collective_point_labels = []
    part_labels, instance_labels = find_part_instances(data, object_instance)

    # Iterate through each part within the current model
    for idx in range(len(part_labels)):
        current_part = part_labels[idx]
        part_indices = np.where(data.y == current_part)

        # Iterate through each instance
        current_inst = 1
        for inst_idx in range(instance_labels[idx]):
            # Find the associated indices for the current part-instance
            points = data.pos[part_indices]
            labels = data.y[part_indices]
            instances = data.instance_label[part_indices]
            instant_indices = np.where(instances == current_inst)

            if display:
                print(f'Looking at {object_instance[part_labels[idx]]}-{current_inst}')
                print(f'{object_instance[part_labels[idx]]}-{current_inst} has {points.shape} points')


            # Select the current part-instance and add it to the list
            points = points[instant_indices]
            labels = labels[instant_indices]
            collective_part_points.append(points)
            collective_point_labels.append(labels)
            current_inst += 1
    return collective_part_points, collective_point_labels


if __name__ == '__main__':
    parser = argparse.ArgumentParser()
    parser.add_argument('--model_root', default=None, help='path to model meshes')
    parser.add_argument('--part_root', default=None, help='path to parts')
    parser.add_argument('--conn_root', default=None, help='path to connectivity graphs')
    args = parser.parse_args()

    PART_ROOT = PART_ROOT if args.part_root is None else args.part_root
    CONNECTIVITY_ROOT = CONNECTIVITY_ROOT if args.conn_root is None else args.conn_root
    MODEL_ROOT = MODEL_ROOT if args.model_root is None else args.model_root

    categorical_instances = categorical_list_of_models()
    list_of_dressers = categorical_instances['dresser']
    known_part_labels = aggregate_part_labels(list_of_dressers)

    get_con_graph_and_bounding_boxes(list_of_dressers[-1], known_part_labels)
    # obj_to_ply_to_shell_script(list_of_dressers)



from graph_articulations.data.process_models import update_part_and_instance_labels
from graph_articulations.data.process_models import categorical_list_of_models
from graph_articulations.data.process_models import aggregate_part_labels
from graph_articulations.data.process_models import get_con_graph_and_bounding_boxes
from graph_articulations.data.process_models import find_part_instances
from graph_articulations.data.process_models import list_of_separated_parts
from graph_articulations.data.dataset import Articulations
from graph_articulations.data.SamplePointsWithLabels import SamplePointsWithLabels





from graph_articulations.data import list_of_separated_parts
from torch import tensor
import torch
from scipy.spatial.distance import directed_hausdorff
from graph_articulations.data import get_con_graph_and_bounding_boxes
import numpy as np

def min_distances(data: object, object_instances: list):
    """
    :param data: Data object containing the current model's information
    :param object_instances: List of all parts within the dataset
    :return:
    """

    from graph_articulations.config import CONNECTIVITY_ROOT
    from graph_articulations import line_br
    import simplejson as json

    conn_path = str(CONNECTIVITY_ROOT) + f'/{129}.artpre.json'
    with open(conn_path, 'r') as json_file:
        precom_dict = json.load(json_file)

    # Grab the part label names and indices
    angels_graph = precom_dict['connectivityGraph']
    conn_graph = torch.zeros(len(data.pids.keys), len(data.pids.keys))
    conn_graph[:][:] = np.nan

    connected_threshold = precom_dict['metadata']['minDist']
    neighboring_parts = precom_dict['metadata']['maxDist']
    ground_truth = precom_dict['parts']

    for idx in range(len(data.pids.keys)):
        cur_conn = conn_graph[idx]
        part = data.parts[f'{idx}']
        label = data.pids[f'{idx}'][0]
        inst = data.pids[f'{idx}'][1]


        # todo construct new list and remove the current key from consideration
        adjacent_parts = {key:val for key, val in data.parts if key != f'{idx}'}
        adjacent_labels = {key:val[0] for key, val in data.pids if key != f'{idx}'}


        if object_instances[label] != 'unknown':
            print(f'{ground_truth[idx]["name"]}')
            print(f'{idx}) {object_instances[label]}-{inst}\n{line_br}')
            for adj_idx in range(len(data.pids.keys)):
                adj_part = data.parts[f'{adj_idx}']
                adj_label = data.pids[f'{adj_idx}'][0]
                adj_inst = data.pids[f'{adj_idx}'][1]

                if adj_idx == idx or object_instances[adj_label] == 'unknown':
                    continue

                dist = max(directed_hausdorff(part.numpy(), adj_part.numpy())[0],
                           directed_hausdorff(adj_part.numpy(), part.numpy())[0])

                print(f'{adj_idx}) {object_instances[adj_label]}-{adj_inst} dist: {dist} ', end="")

                if dist <= connected_threshold:
                    cur_conn[adj_idx] = adj_label
                    conn_graph[adj_idx][idx] = label
                    print(' \tCONNECTED')
                elif dist <= neighboring_parts:
                    cur_conn[adj_idx] = adj_label
                    conn_graph[adj_idx][idx] = label
                    print(' \t HIT')
                else:
                    print()

        conn_graph[idx] = cur_conn
        print(f'Estimated: {conn_graph[idx]}')
        print(f'Converted: {data.connectivity_graph[idx]}')
        print(f'Provided : {angels_graph[idx]}\n')
    #
    # for part, label in zip(part_list, label_list):
    #     cur_conn = conn_graph[idx]
    #
    #     print(f'{idx}) {object_instances[label[0]]}\n{line_br}')
    #     adjacent_part_list, adjacent_label_list = part_list, label_list
    #
    #     adj_idx = 0
    #     for adjacent_part, adjacent_label in zip(adjacent_part_list, label_list):
    #         if torch.equal(part, adjacent_part):
    #             continue
    #
    #         dist = min(directed_hausdorff(part.numpy(), adjacent_part.numpy())[0],
    #                    directed_hausdorff(adjacent_part.numpy(), part.numpy())[0])
    #         if adjacent_label.size == 0 or label.size == 0:
    #             continue
    #
    #         print(f'{adj_idx}) {object_instances[adjacent_label[0]]}', end="")
    #         print(f' dist: {dist} ', end="")
    #
    #         if dist <= threshold and label[0] != adjacent_label[0]:
    #             cur_conn[adj_idx] = adjacent_label[0]
    #             conn_graph[adj_idx][idx] = label[0]
    #             print(' \tCONNECTED')
    #         else:
    #             print()
    #
    #         adj_idx += 1
    #     conn_graph[idx] = cur_conn
    #     print(f'Estimated: {conn_graph[idx]}')
    #     print(f'Converted: {data.connectivity_graph[idx]}')
    #     print(f'Provided : {angels_graph[idx]}\n')
    #     idx += 1

